import base64
import boto3
import sys
import os
import subprocess


class AWS():
    def __init__(self, service, credentials) -> None:
        self.service = service
        self.credentials = credentials
        self.region = 'us-east-1'

    def setup_aws_client(self):
        """Helper function to set up an AWS client for a given service."""

        return boto3.client(
            self.service,
            region_name=self.region,
            aws_access_key_id=self.credentials["AccessKeyId"],
            aws_secret_access_key=self.credentials["SecretAccessKey"],
            aws_session_token=self.credentials["SessionToken"],
        )

    def assume_role(self, sts_client, role_arn, session_name):
        """Helper function to assume an AWS role."""

        return sts_client.assume_role(RoleArn=role_arn, RoleSessionName=session_name)[
            "Credentials"
        ]

    def login_to_ecr(self, image_name):
        """Main function to log into AWS ECR.

        Args:
            image_name (str): Image name used to extract login information from.
        """

        try:
            aws_account = image_name.split(".")[0]
            region = image_name.split(".")[3]

            # Set up the initial AWS clients with psec credentials
            os.environ["AWS_ACCESS_KEY_ID"] = os.environ["AWS_SECPROD_KEY_ID"]
            os.environ["AWS_SECRET_ACCESS_KEY"] = os.environ["AWS_SECPROD_SECRET"]

            sts_client = boto3.client("sts")

            # Assume the psec role in SECPROD
            psec_role_arn = f"arn:aws:iam::{os.environ['AWS_SECPROD_ACCOUNT_ID']}:role/psec-ecr-image-scanner"
            secprod_credentials = self.assume_role(sts_client, psec_role_arn, "ecr-session")

            # Set up the client for the target account using the assumed role credentials
            target_account_role_arn = (
                f"arn:aws:iam::{aws_account}:role/psec-ecr-image-scanner"
            )
            target_sts_client = self.setup_aws_client("sts", secprod_credentials)

            target_credentials = self.assume_role(
                target_sts_client, target_account_role_arn, "ecr-session"
            )

            # Get the ECR credentials using boto3
            ecr_client = self.setup_aws_client("ecr", target_credentials, region)
            docker_password = ecr_client.get_authorization_token()["authorizationData"][0][
                "authorizationToken"
            ]
            docker_password = base64.b64decode(docker_password).decode().split(":")[1]

            # Log into ECR in the target AWS account
            login_command = [
                "docker",
                "login",
                "--username",
                "AWS",
                "--password-stdin",
                f"{aws_account}.dkr.ecr.{region}.amazonaws.com",
            ]

            subprocess.run(
                login_command,
                input=docker_password,
                text=True,
            )

            print("** INFO: Successfully logged into AWS ECR.")
        except Exception as e:
            print(f"** ERROR: An error occurred while logging into AWS ECR: {e}")
            sys.exit(1)
